<?php

namespace Assetise\Scheduler\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Assetise\Scheduler\Forms\Scheduler as SchedulerForm;
use Assetise\Scheduler\Http\Requests\SchedulerResourceRequest;
use Assetise\Scheduler\Http\Resources\SchedulerResource;
use Assetise\Scheduler\Http\Resources\SchedulersCollection;
use Assetise\Scheduler\Models\Scheduler;
use Assetise\Scheduler\Actions\SchedulerAction;
use Assetise\Scheduler\Actions\SchedulerActions;


/**
 * Resource controller class for scheduler.
 */
class SchedulerResourceController extends BaseController
{

    /**
     * Initialize scheduler resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = SchedulerForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('assetise.scheduler.modules'), 'scheduler', guard_url('scheduler'));
            return $next($request);
        });
    }

    /**
     * Display a list of scheduler.
     *
     * @return Response
     */
    public function index(SchedulerResourceRequest $request)
    {
        $request = $request->all();
        $page = SchedulerActions::run('paginate', $request);

        $data = new SchedulersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('scheduler::scheduler.names'))
            ->view('scheduler::scheduler.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display scheduler.
     *
     * @param Request $request
     * @param Model   $scheduler
     *
     * @return Response
     */
    public function show(SchedulerResourceRequest $request, Scheduler $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new SchedulerResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('scheduler::scheduler.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('scheduler::scheduler.show')
            ->output();
    }

    /**
     * Show the form for creating a new scheduler.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SchedulerResourceRequest $request, Scheduler $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new SchedulerResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('scheduler::scheduler.name'))
            ->view('scheduler::scheduler.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new scheduler.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SchedulerResourceRequest $request, Scheduler $model)
    {
        try {
            $request = $request->all();
            $model = SchedulerAction::run('store', $model, $request);
            $data = new SchedulerResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('scheduler::scheduler.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('scheduler/scheduler/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/scheduler/scheduler'))
                ->redirect();
        }

    }

    /**
     * Show scheduler for editing.
     *
     * @param Request $request
     * @param Model   $scheduler
     *
     * @return Response
     */
    public function edit(SchedulerResourceRequest $request, Scheduler $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new SchedulerResource($model);
        // return view('scheduler::scheduler.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('scheduler::scheduler.name'))
            ->view('scheduler::scheduler.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the scheduler.
     *
     * @param Request $request
     * @param Model   $scheduler
     *
     * @return Response
     */
    public function update(SchedulerResourceRequest $request, Scheduler $model)
    {
        try {
            $request = $request->all();
            $model = SchedulerAction::run('update', $model, $request);
            $data = new SchedulerResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('scheduler::scheduler.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('scheduler/scheduler/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('scheduler/scheduler/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the scheduler.
     *
     * @param Model   $scheduler
     *
     * @return Response
     */
    public function destroy(SchedulerResourceRequest $request, Scheduler $model)
    {
        try {

            $request = $request->all();
            $model = SchedulerAction::run('destroy', $model, $request);
            $data = new SchedulerResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('scheduler::scheduler.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('scheduler/scheduler/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('scheduler/scheduler/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
